use strict;
use warnings;
use Test::More;
use lib 't/lib';
use PScm::Test tests => 8;

BEGIN { use_ok('PScm') }

eval_ok(<<EOF, <<EOR, 'built in error');
(define div
  (lambda (numerator denominator)
    (if denominator
        (/ numerator denominator)
        (error "division by zero"))))
(+ (div 2 0) 1)
EOF
div
Error: division by zero
EOR

eval_ok(<<EOF, <<EOR, 'argument to error need not be a string');
(error '(an error "message"))
EOF
Error: (an error "message")
EOR

eval_ok(<<EOF, <<EOR, 'internal type error and recovery');
(* 2 "2")
(* 2 2)
EOF
Error: wrong type argument(PScm::Expr::String) to PScm::Primitive::Multiply
4
EOR

eval_ok(<<EOF, <<EOR, 'internal lookup error and recovery');
x
2
EOF
Error: no binding for x in PScm::Env
2
EOR

eval_ok(<<EOF, <<EOR, 'method lookup error and recovery');
(define testclass
  (make-class
    root
    ()
    (say-hello () 'hello)))
(define testobj (testclass))
(testobj say-goodbye)
(testobj say-hello)
EOF
testclass
testobj
Error: method say-goodbye not found
hello
EOR

eval_ok(<<EOF, <<EOR, 'set! error and recovery');
(set! x 1)
2
EOF
Error: no binding for x in PScm::Env
2
EOR

eval_ok(<<EOF, <<EOR, 'lambda error and recovery');
(define test
  (lambda (a b c)
    (list a b c)))
(test 1 2)
2
EOF
test
Error: not enough arguments
2
EOR

# vim: ft=perl
